# -*- encoding: utf-8 -*-
##############################################################################
#
#    Copyright (C) 2009 Albert Cervera i Areny (<http://www.nan-tic.com>). All Rights Reserved
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from xml import dom, xpath
from osv import osv, fields

class ir_documentation_paragraph(osv.osv):
    _name = 'ir.documentation.paragraph'
    _rec_name = 'identifier'
    _order = 'parent_left,sequence'
    _parent_name = "parent_id"
    _parent_order = 'id'
    _parent_store = True

    def _find_views(self, cr, uid, text, context=None):
        view_ids = []
        root = dom.minidom.parseString( '<html>%s</html>' % text )
        for x in xpath.Evaluate('//view/@name', root):
            module, dot, view = x.nodeValue.rpartition('.')
            module = module.strip()
            view = view.strip()
            ids = self.pool.get('ir.model.data').search(cr, uid, [('module','=',module),('model','=','ir.ui.view'),('name','=',view)], context=context)
            data = self.pool.get('ir.model.data').read(cr, uid, ids, ['res_id'], context)
            view_ids += [x['res_id'] for x in data]
        return list(set(view_ids))

    def _view_ids(self, cr, uid, ids, name, arg, context=None):
        result = {}
        for paragraph in self.browse(cr, uid, ids, context):
            result[paragraph.id] = self._find_views(cr, uid, paragraph.text, context)
        return result

    def _find_fields(self, cr, uid, text, context=None):
        field_ids = []
        root = dom.minidom.parseString( '<html>%s</html>' % text )
        for x in xpath.Evaluate('//field/@name', root):
            model, dot, field = x.nodeValue.rpartition('.')
            model = model.strip()
            field = field.strip()
            ids = self.pool.get('ir.model.fields').search(cr, uid, [('model','=',model),('name','=',field)], context=context)
            field_ids += ids
        return list(set(field_ids))

    def _field_ids(self, cr, uid, ids, name, arg, context=None):
        result = {}
        for paragraph in self.browse(cr, uid, ids, context):
            result[paragraph.id] = self._find_fields(cr, uid, paragraph.text, context)
        return result

    def replace(self, root, tag, newtag):
        for x in xpath.Evaluate(tag, root):
            x.localName = newtag

    def _parse(self, cr, uid, id, text, context=None):
        root = dom.minidom.parseString( '<html>%s</html>' % text )
        #self.replace(root, '//field', 'i')
        xml = root.toxml(encoding="utf-8")
        return xml.replace('<html>', '').replace('</html>','')

    def _render(self, cr, uid, id, text, image, type, context=None):
        text = text or ''
        text = self._parse(cr, uid, id, text, context)
        image = image or ''
        if type == 'title':
            return '<h1>%s</h1>' % text
        elif type == 'chapter':
            return '<h2>%s</h2>' % text
        elif type == 'section':
            return '<h2>%s</h2>' % text
        elif type == 'subsection':
            return '<h3>%s</h3>' % text
        elif type == 'subsubsection':
            return '<h4>%s</h4>' % text
        elif type == 'paragraph':
            return '<p>%s</p>' % text
        elif type == 'note':
            return '<div class="note"><p>%s</p></div>' % text
        elif type == 'image':
            return '<div class="image"><img src="data:image/gif;base64,%s"/><p>%s</p></div>' % (image, text)
        else:
            return text

    def _html(self, cr, uid, ids, name, arg, context=None):
        result = {}
        for paragraph in self.browse(cr, uid, ids, context):
            result[paragraph.id] = self._render(cr, uid, paragraph.id, paragraph.text, paragraph.image, paragraph.type, context)
        return result

    def _all_html(self, cr, uid, ids, name, arg, context=None):
        result = {}
        for id in ids:
            text = ''
            all_ids = self.search(cr, uid, [('parent_id','child_of',[id])], context=context)
            for paragraph in self.browse(cr, uid, all_ids, context):
                text += paragraph.html
                #if paragraph.id == id:
                    #text += '<div style="border-style: dotted; border-width: 1px; color: red; padding: 0px 0px; margin: 0px 0px;">%s</div>' % paragraph.html
                #else:
            result[id] = text
        return result

    _columns = {
        'identifier': fields.char('Identifier', size=200, required=True, help='Unique internal identifier'),
        'module': fields.many2one('ir.module.module', 'Module', required=True, ondelete='cascade'),
        'text': fields.text('Text', help='Paragraph body'),
        'type': fields.selection([('title','Title'),('paragraph','Paragraph'),
            ('chapter','Chapter'),('section','Section'),('subsection','Subsection'),
            ('subsubsection','Subsubsection'),('note','Note'),('image','Image')], 
            'Paragraph Type', required=True),
        'inherit_id': fields.many2one('ir.documentation.paragraph', 'Inherit'),
        'position': fields.selection([('before','Before'),('after','After'),('replace','Replace')], 'Position'),
        'image': fields.binary('Image'),
        'parent_id': fields.many2one('ir.documentation.paragraph', 'Parent Paragraph', ondelete='cascade'),
        'child_ids': fields.one2many('ir.documentation.paragraph', 'parent_id', 'Children Paragraphs'),
        'parent_left': fields.integer('Parent Left', select=1),
        'parent_right': fields.integer('Parent Right', select=1),
        'sequence': fields.integer('Sequence', required=True),
        'html': fields.function(_html, method=True, string='HTML', type='text', store=True, help="Rendered text"),
        'all_html': fields.function(_all_html, method=True, string='HTML', type='text', help="All rendered text"),
        'field_ids': fields.function(_field_ids, method=True, string='Referenced Fields', type='many2many', relation='ir.model.fields', help='Fields referenced in the paragraph.'),
        'view_ids': fields.function(_view_ids, method=True, string='Referenced Views', type='many2many', relation='ir.ui.view', help='Views referenced in the paragraph.'),
    }
ir_documentation_paragraph()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
